"""
This module is used to plot the metrics of the network adapter on the cli
"""

import plotext as plt
import time
from typing import List
from datetime import datetime
from libs.metrics import Metrics
from libs.session import SessionMetrics
import os


class MetricsPlotter:
    def __init__(self, max_points: int = 50):
        self.signal_strengths: List[float] = []
        self.timestamps: List[str] = []
        self.max_points = max_points
        self.min_seen = 0
        self.max_seen = -100
        self.session = SessionMetrics()

        # Create plots directory if it doesn't exist
        os.makedirs("plots", exist_ok=True)

    def save_plot(self):
        """Save the current plot with averages"""
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = f"plots/sigmon_{timestamp}.png"

        # Calculate average
        avg_signal = sum(self.signal_strengths) / len(self.signal_strengths)

        # Create the final plot
        plt.clf()
        plt.plotsize(150, 50)  # Larger size for better image quality

        # Calculate dynamic y-axis limits
        y_min = max(-100, self.min_seen - 10)
        y_max = min(0, self.max_seen + 10)
        plt.ylim(y_min, y_max)

        # Set x-axis limits
        x_max = len(self.signal_strengths)
        plt.xlim(0, x_max)

        # Plot signal strength
        plt.plot(self.signal_strengths, marker="dot", color="green", label="Signal")

        # Plot average line
        avg_line = [avg_signal] * len(self.signal_strengths)
        plt.plot(avg_line, color="red", label=f"Avg: {avg_signal:.1f} dBm")

        plt.theme("matrix")
        plt.title("WiFi Signal Strength Over Time")
        plt.xlabel("Time (seconds)")
        plt.ylabel("Signal Strength (dBm)")

        # Build and save the plot
        plt.build()
        plt.save_fig(filename)

        print(f"\nPlot saved as: {filename}")

    def update_plot(self, metrics):
        # Convert signal strength to float and ensure it's negative
        try:
            signal = float(metrics.signal_strength)
            # Update min/max seen values
            self.min_seen = min(self.min_seen, signal)
            self.max_seen = max(self.max_seen, signal)
        except ValueError:
            print(f"Warning: Invalid signal strength value: {metrics.signal_strength}")
            signal = 0

        current_time = datetime.now().strftime("%H:%M:%S")

        # Add new data points
        self.signal_strengths.append(signal)
        self.timestamps.append(current_time)

        # Keep only last max_points
        if len(self.signal_strengths) > self.max_points:
            self.signal_strengths.pop(0)
            self.timestamps.pop(0)

        # Clear the terminal
        plt.clear_terminal()

        # Create the plot
        plt.clf()
        plt.plotsize(100, 30)

        # Calculate dynamic y-axis limits
        y_min = max(-100, self.min_seen - 10)  # Don't go below -100
        y_max = min(0, self.max_seen + 10)  # Don't go above 0
        plt.ylim(y_min, y_max)

        # Set x-axis limits
        x_max = len(self.signal_strengths)
        plt.xlim(0, x_max)

        # Plot with a thicker line
        # Color depends on the signal strength:
        # - Red if signal strength is less than -60
        # - Orange if signal strength is less than -50
        # - Green if signal strength is greater than -50
        signal_color = "red" if signal < -60 else "orange" if signal < -50 else "green"
        plt.plot(self.signal_strengths, color="green")
        plt.theme("matrix")
        plt.title("WiFi Signal Strength Over Time")
        plt.xlabel("Time (seconds)")
        plt.ylabel("Signal Strength (dBm)")

        # Show the plot
        plt.show()

        # Print current metrics below the plot
        print(
            f"Signal: {signal:.1f} dBm | Bitrate: {metrics.bitrate} Mb/s | Power Save: {'On' if metrics.is_power_save_enabled else 'Off'}"
        )
        status = (
            "Strong signal"
            if signal > -50
            else (
                "Good signal"
                if signal > -60
                else "Weak signal" if signal > -70 else "Very bad signal"
            )
        )
        print(f"Status: {status}")
        summary = self.session.get_session_summary()
        print(summary)

        self.session.add_metrics(metrics)


def plot_metrics_live(
    get_metrics_func, adapter_name: str = "wlan0", interval: float = 1.0
):
    """
    Continuously plot metrics in real-time

    Args:
        get_metrics_func: Function to get metrics
        adapter_name: Name of the network adapter
        interval: Update interval in seconds
    """
    plotter = MetricsPlotter()

    try:
        while True:
            metrics = get_metrics_func(adapter_name)
            plotter.update_plot(metrics)
            time.sleep(interval)
    except KeyboardInterrupt:
        print("\nStopping metrics plotting...")
        print(plotter.session.get_session_summary())
        plotter.save_plot()
