import os
import sys
import subprocess
import argparse
from libs.metrics import Metrics
from libs.plotter import plot_metrics_live


def get_metrics(adapter_name: str = "wlan0") -> Metrics:
    """
    Get the metrics of the network adapter
    """
    # Execute iwconfig command and return the output
    iwconfig_output = subprocess.run(
        ["iwconfig", adapter_name], capture_output=True, text=True
    )

    output = iwconfig_output.stdout

    # Initialize default values
    signal_strength = "0"
    bitrate = "0"
    is_power_save_enabled = False

    # Try different possible formats for signal level
    try:
        if "Signal level=" in output:
            signal_strength = output.split("Signal level=")[1].split(" dBm")[0].strip()
        elif "Signal level" in output:
            signal_strength = output.split("Signal level")[1].split("dBm")[0].strip()
        elif "signal:" in output.lower():
            signal_strength = output.lower().split("signal:")[1].split("dBm")[0].strip()

        # Try different possible formats for bit rate
        if "Bit Rate=" in output:
            bitrate = output.split("Bit Rate=")[1].split(" Mb/s")[0].strip()
        elif "Bit Rate:" in output:
            bitrate = output.split("Bit Rate:")[1].split(" Mb/s")[0].strip()
        elif "tx bitrate:" in output.lower():
            bitrate = output.lower().split("tx bitrate:")[1].split("mbit/s")[0].strip()

        # Try different possible formats for power management
        if "Power Management:" in output:
            is_power_save_enabled = (
                "on" in output.split("Power Management:")[1].lower().strip()
            )
        elif "power management" in output.lower():
            is_power_save_enabled = (
                "on" in output.lower().split("power management")[1].strip()
            )

    except (IndexError, ValueError) as e:
        print(f"Warning: Error parsing iwconfig output: {e}")
        print(f"Raw output: {output}")

    return Metrics(signal_strength, bitrate, is_power_save_enabled)


def get_active_interface():
    """
    Get the active interface name
    """
    all_interfaces = subprocess.run(["iwconfig"], capture_output=True, text=True)
    for interface in all_interfaces.stdout.split("\n"):
        if "ESSID" in interface:
            return interface.split(" ")[0].strip()
    return None


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--interface", type=str, help="The interface to use")
    args = parser.parse_args()  # Check if we got an interface name
    if not args.interface:
        print("No interface name provided: we will try to find the active interface")
        try:
            active_interface = get_active_interface()
            print(f"The active interface is {active_interface}")
        except Exception as e:
            print(f"Error getting active interface: {e}")
            print("Using wlan0 as default interface")
            active_interface = "wlan0"
    else:
        print(f"Using interface {args.interface}")
        active_interface = args.interface
    # Execute iwconfig command
    metrics = get_metrics(active_interface)
    print(metrics)
    try:
        plot_metrics_live(get_metrics, active_interface)
    except KeyboardInterrupt:
        print("[+] Done")
